@@ -2,6 +2,7 @@ import mon  # noqa
 import pkg  # noqa
 from install import install, mirror_install, repo_install  # noqa
 from uninstall import uninstall  # noqa
+from ceph_deploy.util import pkg_managers
 
 # Allow to set some information about this distro
 #
@@ -19,3 +20,7 @@ def choose_init():
     if distro.lower() == 'ubuntu':
         return 'upstart'
     return 'sysvinit'
+
+
+def get_packager(module):
+    return pkg_managers.Apt(module)
