@@ -1,9 +1,17 @@
 from ceph_deploy.lib import remoto
 from ceph_deploy.hosts.centos.install import repo_install, mirror_install  # noqa
-from ceph_deploy.hosts.util import install_yum_priorities
+from ceph_deploy.util.paths import gpg
+from ceph_deploy.hosts.common import map_components
 
 
-def install(distro, version_kind, version, adjust_repos):
+NON_SPLIT_PACKAGES = ['ceph-osd', 'ceph-mon', 'ceph-mds']
+
+
+def install(distro, version_kind, version, adjust_repos, **kw):
+    packages = map_components(
+        NON_SPLIT_PACKAGES,
+        kw.pop('components', [])
+    )
     logger = distro.conn.logger
     release = distro.release
     machine = distro.machine_type
@@ -14,17 +22,14 @@ def install(distro, version_kind, version, adjust_repos):
         key = 'autobuild'
 
     if adjust_repos:
-        install_yum_priorities(distro)
+        if distro.packager.name == 'yum':
+            distro.packager.install('yum-plugin-priorities')
+            # haven't been able to determine necessity of check_obsoletes with DNF
+            distro.conn.remote_module.enable_yum_priority_obsoletes()
+            logger.warning('check_obsoletes has been enabled for Yum priorities plugin')
 
         if version_kind != 'dev':
-            remoto.process.run(
-                distro.conn,
-                [
-                    'rpm',
-                    '--import',
-                    "https://ceph.com/git/?p=ceph.git;a=blob_plain;f=keys/{key}.asc".format(key=key)
-                ]
-            )
+            distro.packager.add_repo_gpg_key(gpg.url(key))
 
             if version_kind == 'stable':
                 url = 'http://ceph.com/rpm-{version}/fc{release}/'.format(
@@ -51,6 +56,11 @@ def install(distro, version_kind, version, adjust_repos):
                 ]
             )
 
+            # set the right priority
+            logger.warning('ensuring that /etc/yum.repos.d/ceph.repo contains a high priority')
+            distro.conn.remote_module.set_repo_priority(['Ceph', 'Ceph-noarch', 'ceph-source'])
+            logger.warning('altered ceph.repo priorities to contain: priority=1')
+
         if version_kind == 'dev':
             logger.info('skipping install of ceph-release package')
             logger.info('repo file will be created manually')
@@ -60,23 +70,11 @@ def install(distro, version_kind, version, adjust_repos):
                     release=release.split(".", 1)[0],
                     machine=machine,
                     version=version),
-                "https://ceph.com/git/?p=ceph.git;a=blob_plain;f=keys/{key}.asc".format(key=key),
+                gpg.url(key),
                 adjust_repos=True,
                 extra_installs=False
             )
 
-        # set the right priority
-        logger.warning('ensuring that /etc/yum.repos.d/ceph.repo contains a high priority')
-        distro.conn.remote_module.set_repo_priority(['Ceph', 'Ceph-noarch', 'ceph-source'])
-        logger.warning('altered ceph.repo priorities to contain: priority=1')
-
-    remoto.process.run(
-        distro.conn,
-        [
-            'yum',
-            '-y',
-            '-q',
-            'install',
-            'ceph',
-        ],
+    distro.packager.install(
+        packages
     )
