@@ -1,13 +1,12 @@
-import argparse
 import json
 import logging
 import re
 import os
-from textwrap import dedent
 import time
 
 from ceph_deploy import conf, exc, admin
 from ceph_deploy.cliutil import priority
+from ceph_deploy.util.help_formatters import ToggleRawTextHelpFormatter
 from ceph_deploy.util import paths, net, files
 from ceph_deploy.lib import remoto
 from ceph_deploy.new import new_mon_keyring
@@ -58,7 +57,7 @@ def catch_mon_errors(conn, logger, hostname, cfg, args):
     and warn apropriately about it.
     """
     monmap = mon_status_check(conn, logger, hostname, args).get('monmap', {})
-    mon_initial_members = cfg.safe_get('global', 'mon_initial_members')
+    mon_initial_members = get_mon_initial_members(args, _cfg=cfg)
     public_addr = cfg.safe_get('global', 'public_addr')
     public_network = cfg.safe_get('global', 'public_network')
     mon_in_monmap = [
@@ -173,8 +172,7 @@ def concatenate_keyrings(args):
 def mon_add(args):
     cfg = conf.ceph.load(args)
 
-    if not args.mon:
-        raise exc.NeedHostError()
+    # args.mon is a list with only one entry
     mon_host = args.mon[0]
 
     try:
@@ -187,7 +185,7 @@ def mon_add(args):
         )
 
     LOG.info('ensuring configuration of new mon host: %s', mon_host)
-    args.client = [mon_host]
+    args.client = args.mon
     admin.admin(args)
     LOG.debug(
         'Adding mon to cluster %s, host %s',
@@ -235,11 +233,7 @@ def mon_create(args):
 
     cfg = conf.ceph.load(args)
     if not args.mon:
-        mon_initial_members = cfg.safe_get('global', 'mon_initial_members')
-        args.mon = re.split(r'[,\s]+', mon_initial_members)
-
-    if not args.mon:
-        raise exc.NeedHostError()
+        args.mon = get_mon_initial_members(args, error_on_empty=True, _cfg=cfg)
 
     if args.keyrings:
         monitor_keyring = concatenate_keyrings(args)
@@ -398,13 +392,10 @@ def mon_destroy(args):
 
 
 def mon_create_initial(args):
-    cfg = conf.ceph.load(args)
-    cfg_initial_members = cfg.safe_get('global', 'mon_initial_members')
-    if cfg_initial_members is None:
-        raise RuntimeError('No `mon initial members` defined in config')
-    mon_initial_members = re.split(r'[,\s]+', cfg_initial_members)
+    mon_initial_members = get_mon_initial_members(args, error_on_empty=True)
 
     # create them normally through mon_create
+    args.mon = mon_initial_members
     mon_create(args)
 
     # make the sets to be able to compare late
@@ -460,71 +451,68 @@ def mon(args):
 @priority(30)
 def make(parser):
     """
-    Deploy ceph monitor on remote hosts.
+    Ceph MON Daemon management
     """
-    sub_command_help = dedent("""
-    Subcommands:
+    parser.formatter_class = ToggleRawTextHelpFormatter
 
-    create-initial
-      Will deploy for monitors defined in `mon initial members`, wait until
-      they form quorum and then gatherkeys, reporting the monitor status along
-      the process. If monitors don't form quorum the command will eventually
-      time out.
+    mon_parser = parser.add_subparsers(dest='subcommand')
 
-    create
-      Deploy monitors by specifying them like:
-
-        ceph-deploy mon create node1 node2 node3
-
-      If no hosts are passed it will default to use the `mon initial members`
-      defined in the configuration.
-
-    add
-      Add a monitor to an existing cluster:
-
-        ceph-deploy mon add node1
-
-      Or:
-
-        ceph-deploy mon add node1 --address 192.168.1.10
-
-      If the section for the monitor exists and defines a `mon addr` that
-      will be used, otherwise it will fallback by resolving the hostname to an
-      IP. If `--address` is used it will override all other options.
-
-    destroy
-      Completely remove monitors on a remote host. Requires hostname(s) as
-      arguments.
-    """)
-    parser.formatter_class = argparse.RawDescriptionHelpFormatter
-    parser.description = sub_command_help
-
-    parser.add_argument(
-        'subcommand',
-        choices=[
-            'add',
-            'create',
-            'create-initial',
-            'destroy',
-            ],
-        )
-
-    parser.add_argument(
+    mon_add = mon_parser.add_parser(
+        'add',
+        help=('R|Add a monitor to an existing cluster:\n'
+              '\tceph-deploy mon add node1\n'
+              'Or:\n'
+              '\tceph-deploy mon add --address 192.168.1.10 node1\n'
+              'If the section for the monitor exists and defines a `mon addr` that\n'
+              'will be used, otherwise it will fallback by resolving the hostname to an\n'
+              'IP. If `--address` is used it will override all other options.')
+    )
+    mon_add.add_argument(
         '--address',
         nargs='?',
-        dest='address',
+    )
+    mon_add.add_argument(
+        'mon',
+        nargs=1,
     )
 
-    parser.add_argument(
+    mon_create = mon_parser.add_parser(
+        'create',
+        help=('R|Deploy monitors by specifying them like:\n'
+              '\tceph-deploy mon create node1 node2 node3\n'
+              'If no hosts are passed it will default to use the\n'
+              '`mon initial members` defined in the configuration.')
+    )
+    mon_create.add_argument(
+        '--keyrings',
+        nargs='?',
+        help='concatenate multiple keyrings to be seeded on new monitors',
+    )
+    mon_create.add_argument(
+        'mon',
+        nargs='*',
+    )
+
+    mon_create_initial = mon_parser.add_parser(
+        'create-initial',
+        help=('Will deploy for monitors defined in `mon initial members`, '
+              'wait until they form quorum and then gatherkeys, reporting '
+              'the monitor status along the process. If monitors don\'t form '
+              'quorum the command will eventually time out.')
+    )
+    mon_create_initial.add_argument(
         '--keyrings',
         nargs='?',
-        dest='keyrings',
         help='concatenate multiple keyrings to be seeded on new monitors',
     )
 
-    parser.add_argument(
+    mon_destroy = mon_parser.add_parser(
+        'destroy',
+        help='Completely remove Ceph MON from remote host(s)'
+    )
+    mon_destroy.add_argument(
         'mon',
-        nargs='*',
+        nargs='+',
     )
 
     parser.set_defaults(
@@ -536,6 +524,26 @@ def make(parser):
 #
 
 
+def get_mon_initial_members(args, error_on_empty=False, _cfg=None):
+    """
+    Read the Ceph config file and return the value of mon_initial_members
+    Optionally, a NeedHostError can be raised if the value is None.
+    """
+    if _cfg:
+        cfg = _cfg
+    else:
+        cfg = conf.ceph.load(args)
+    mon_initial_members = cfg.safe_get('global', 'mon_initial_members')
+    if not mon_initial_members:
+        if error_on_empty:
+            raise exc.NeedHostError(
+                'could not find `mon initial members` defined in ceph.conf'
+            )
+    else:
+        mon_initial_members = re.split(r'[,\s]+', mon_initial_members)
+    return mon_initial_members
+
+
 def is_running(conn, args):
     """
     Run a command to check the status of a mon, return a boolean.
